# Copyright 2021-2025 Avaiga Private Limited
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
# an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
# specific language governing permissions and limitations under the License.

import keyword

from ..exceptions.exceptions import InvalidConfigurationId

__INVALID_TAIPY_ID_TERMS = ["CYCLE", "SCENARIO", "SEQUENCE", "TASK", "DATANODE"]


def _validate_id(name: str):
    for invalid__id_term in __INVALID_TAIPY_ID_TERMS:
        if invalid__id_term in name:
            raise InvalidConfigurationId(f"'{name}' is not a valid identifier. '{invalid__id_term}' is restricted.")

    if name.isidentifier() and not keyword.iskeyword(name):
        return name

    raise InvalidConfigurationId(f"'{name}' is not a valid identifier.")
